import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';

enum ButtonType { text, elevated, outlined, icon }

class ButtonConfig extends Equatable {
  final Widget child;
  final FutureOr<void> Function()? onPressed;
  final ButtonType type;
  final Color? color;

  const ButtonConfig({
    required this.child,
    this.onPressed,
    this.type = ButtonType.elevated,
    this.color,
  });

  factory ButtonConfig.text(
    String text, {
    FutureOr<void> Function()? onPressed,
    Color? color,
    ButtonType type = ButtonType.text,
  }) {
    return ButtonConfig(
      child: Text(text),
      onPressed: onPressed,
      color: color,
      type: type,
    );
  }

  @override
  List<Object?> get props => [child, onPressed, type, color];

  ButtonConfig copyWith({
    Widget? child,
    FutureOr<void> Function()? onPressed,
    ButtonType? type,
    Color? color,
  }) {
    return ButtonConfig(
      child: child ?? this.child,
      onPressed: onPressed ?? this.onPressed,
      type: type ?? this.type,
      color: color ?? this.color,
    );
  }
}
