import 'dart:async';

abstract class ListPopulator<T> {
  final String Function(T)? _asString;
  final String Function(T)? _asValue;

  ListPopulator({
    String Function(T)? asString,
    String Function(T)? asValue,
  })  : _asString = asString,
        _asValue = asValue;

  FutureOr<List<T>> getItems(String text);

  String asString(T item) {
    return _asString?.call(item) ?? item.toString();
  }

  String asValue(T item) {
    return _asValue?.call(item) ?? asString(item);
  }
}

class StaticListPopulator<T> extends ListPopulator<T> {
  final List<T> items;

  StaticListPopulator(
    this.items, {
    super.asString,
    super.asValue,
  });

  @override
  List<T> getItems(String text) {
    return items.where((item) {
      return asString(item).toLowerCase().contains(text.toLowerCase());
    }).toList();
  }
}

class DynamicListPopulator<T> extends ListPopulator<T> {
  final FutureOr<List<T>> Function(String text) onLoad;

  DynamicListPopulator(
    this.onLoad, {
    super.asString,
    super.asValue,
  });

  @override
  FutureOr<List<T>> getItems(String text) {
    return onLoad(text);
  }
}
